@@ -19,11 +19,11 @@ import hashlib
 import json
 import logging
 import os
+import os.path
 import signal
 import socket
 
 import aiohttp
-import requests
 
 from . import core
 from . import mgmt
@@ -57,6 +57,20 @@ class HSAPIClient:
             self.default_key_index = None
             self._cached_key = None
         self.current_wdeployment = None
+        self.loop.run_until_complete(self.async_init())
+
+
+    async def async_init(self):
+        if self.verify_certs:
+            self.session = aiohttp.ClientSession(headers=self._add_key_header())
+        else:
+            conn = aiohttp.TCPConnector(ssl=False)
+            self.session = aiohttp.ClientSession(connector=conn, headers=self._add_key_header())
+
+
+    def __del__(self):
+        self.loop.run_until_complete(self.session.close())
+
 
     def _add_key_header(self, headers=None):
         if headers is None:
@@ -67,15 +81,15 @@ class HSAPIClient:
 
     def get_remote_config(self):
         headers = self._add_key_header()
-        res = requests.get(self.base_uri + '/list',
-                           headers=headers, verify=self.verify_certs)
-        if res.ok:
-            return res.json()
-        elif res.status_code == 400:
-            return {'err': res.json()['error_message']}
+        res = self.loop.run_until_complete(self.session.get(self.base_uri + '/list'))
+        if res.status == 200:
+            return self.loop.run_until_complete(res.json())
+        elif res.status == 400:
+            err = self.loop.run_until_complete(res.json())['error_message']
+            return {'err': err}
         else:
             raise Error('error contacting hardshare server: {}'
-                        .format(res.status_code))
+                        .format(res.status))
 
     def sync_config(self):
         mgmt.modify_local(self.local_config)
@@ -94,11 +108,9 @@ class HSAPIClient:
         if at_most_one and len(self.local_config['wdeployments']) > 0:
             raise Error('local configuration already declares a'
                         ' workspace deployment (and at_most_one=True)')
-        headers = self._add_key_header()
-        res = requests.post(self.base_uri + '/register',
-                            headers=headers, verify=self.verify_certs)
-        if res.ok:
-            payload = res.json()
+        res = self.loop.run_until_complete(self.session.post(self.base_uri + '/register'))
+        if res.status == 200:
+            payload = self.loop.run_until_complete(res.json())
             assert 'id' in payload
             self.local_config['wdeployments'].append({
                 'id': payload['id'],
@@ -107,20 +119,19 @@ class HSAPIClient:
             self.sync_config()
         else:
             raise Error('error contacting hardshare server: {}'
-                        .format(res.status_code))
+                        .format(res.status))
         return payload['id']
 
     def declare_existing(self, id_prefix):
-        headers = self._add_key_header()
-        res = requests.get(self.base_uri + '/list',
-                           headers=headers, verify=self.verify_certs)
-        if res.ok:
-            payload = res.json()
+        res = self.loop.run_until_complete(self.session.get(self.base_uri + '/list'))
+        if res.status == 200:
+            payload = self.loop.run_until_complete(res.json())
         elif res.status_code == 400:
-            return {'err': res.json()['error_message']}
+            err = self.loop.run_until_complete(res.json())['error_message']
+            return {'err': err}
         else:
             raise Error('error contacting hardshare server: {}'
-                        .format(res.status_code))
+                        .format(res.status))
         for wd in payload['deployments']:
             if wd['id'].startswith(id_prefix):
                 id_prefix = wd['id']
@@ -145,19 +156,18 @@ class HSAPIClient:
                 raise ValueError(msg)
             else:
                 id_prefix = self.local_config['wdeployments'][0]['id']
-        headers = self._add_key_header()
-        res = requests.get(self.base_uri + '/check/{}'.format(id_prefix),
-                           headers=headers, verify=self.verify_certs)
-        if res.ok:
+        res = self.loop.run_until_complete(self.session.get(self.base_uri + '/check/{}'.format(id_prefix)))
+        if res.status == 200:
             pass
-        elif res.status_code == 404:
+        elif res.status == 404:
             return {'err': 'not found', 'id_prefix': id_prefix}
-        elif res.status_code == 400:
-            return {'err': res.json()['error_message'], 'id_prefix': id_prefix}
+        elif res.status == 400:
+            err = self.loop.run_until_complete(res.json())['error_message']
+            return {'err': err, 'id_prefix': id_prefix}
         else:
             raise Error('error contacting hardshare server: {}'
-                        .format(res.status_code))
-        return res.json()
+                        .format(res.status))
+        return self.loop.run_until_complete(res.json())
 
     def dissolve_registration(self, id_prefix=None):
         if id_prefix is None:
@@ -167,19 +177,18 @@ class HSAPIClient:
                 raise ValueError(msg)
             else:
                 id_prefix = self.local_config['wdeployments'][0]['id']
-        headers = self._add_key_header()
-        res = requests.post(self.base_uri + '/dis/{}'.format(id_prefix),
-                            headers=headers, verify=self.verify_certs)
-        if res.ok:
+        res = self.loop.run_until_complete(self.session.post(self.base_uri + '/dis/{}'.format(id_prefix)))
+        if res.status == 200:
             pass
-        elif res.status_code == 404:
+        elif res.status == 404:
             return {'err': 'not found', 'id_prefix': id_prefix}
-        elif res.status_code == 400:
-            return {'err': res.json()['error_message'], 'id_prefix': id_prefix}
+        elif res.status == 400:
+            err = self.loop.run_until_complete(res.json())['error_message']
+            return {'err': err, 'id_prefix': id_prefix}
         else:
             raise Error('error contacting hardshare server: {}'
-                        .format(res.status_code))
-        return res.json()
+                        .format(res.status))
+        return self.loop.run_until_complete(res.json())
 
     def terminate(self, wdid=None):
         if wdid is None:
@@ -296,6 +305,20 @@ class HSAPIClient:
 
         if payload['cmd'] == 'INSTANCE_LAUNCH':
             if self.current is None:
+                if payload['ct'] == 'sshtun':
+                    if 'ssh_key' not in self.local_config or self.local_config['ssh_key'] is None:
+                        logger.warning('request to launch with connection type sshtun, '
+                                       'but local configuration declares no SSH key')
+                        logger.debug('in response to INSTANCE_LAUNCH, sending NACK')
+                        await ws.send_str(json.dumps({
+                            'v': 0,
+                            'cmd': 'NACK',
+                            'mi': payload['mi'],
+                        }))
+                        return True
+                    if not os.path.exists(self.local_config['ssh_key']):
+                        logger.warning('key path for sshtun appears to not exist: {}'.format(self.local_config['ssh_key']))
+
                 self.ws_recvmap[payload['id']] = asyncio.Queue()
                 cprovider = self.current_wdeployment['cprovider']
                 cargs = self.current_wdeployment.get('cargs', None)
@@ -318,6 +341,7 @@ class HSAPIClient:
                     'cmd': 'ACK',
                     'mi': payload['mi'],
                 }))
+
             else:
                 logger.debug('in response to INSTANCE_LAUNCH, sending NACK')
                 await ws.send_str(json.dumps({
