@@ -24,7 +24,7 @@ import subprocess
 import sys
 
 import yaml
-from requests.exceptions import ConnectionError
+from aiohttp.client_exceptions import ClientConnectorError as ConnectionError
 
 from .core import WorkspaceInstance
 from .mgmt import get_local_config, add_key, add_ssh_path, list_local_keys
@@ -34,16 +34,6 @@ from .err import Error as HSError
 from .addons import camera_main
 
 
-logger = logging.getLogger('hardshare')
-logger.setLevel(logging.WARNING)
-loghandler = logging.handlers.WatchedFileHandler(filename='hardshare_client.log', mode='a', delay=True)
-loghandler.setLevel(logging.DEBUG)
-loghandler.setFormatter(logging.Formatter('%(name)s.%(funcName)s (%(levelname)s) (pid: {});'
-                                          ' %(asctime)s ; %(message)s'
-                                          .format(os.getpid())))
-logger.addHandler(loghandler)
-
-
 def get_config_with_index(id_prefix=None):
     try:
         config = get_local_config()
@@ -82,6 +72,15 @@ def get_config_with_index(id_prefix=None):
 
 
 def main(argv=None):
+    pkglogger = logging.getLogger('hardshare')
+    pkglogger.setLevel(logging.WARNING)
+    loghandler = logging.handlers.WatchedFileHandler(filename='hardshare_client.log', mode='a', delay=True)
+    loghandler.setLevel(logging.DEBUG)
+    loghandler.setFormatter(logging.Formatter('%(name)s.%(funcName)s (%(levelname)s) (pid: {});'
+                                              ' %(asctime)s ; %(message)s'
+                                              .format(os.getpid())))
+    pkglogger.addHandler(loghandler)
+
     if argv is None:
         argv = sys.argv[1:]
     argparser = argparse.ArgumentParser(description=('Command-line interface'
@@ -143,6 +142,9 @@ def main(argv=None):
     config_parser.add_argument('--add-raw-device', metavar='PATH', type=str,
                                dest='raw_device_path', default=None,
                                help='add device file to present in container')
+    config_parser.add_argument('--cprovider', metavar='CPROVIDER', type=str,
+                               dest='cprovider', default=None,
+                               help='select a container provider: docker, podman')
     config_parser.add_argument('--assign-image', metavar='IMG', type=str,
                                dest='cprovider_img', default=None,
                                help='assign image for cprovider to use (advanced option)')
@@ -297,7 +299,7 @@ def main(argv=None):
         return 0
 
     if argv_parsed.verbose:
-        logger.setLevel(logging.DEBUG)
+        pkglogger.setLevel(logging.DEBUG)
 
     if argv_parsed.output_format is not None:
         output_format = argv_parsed.output_format.lower()
@@ -375,6 +377,10 @@ def main(argv=None):
         config, index, rc = get_config_with_index(argv_parsed.id_prefix)
         if rc != 0:
             return rc
+        if 'ssh_key' not in config or config['ssh_key'] is None:
+            print('WARNING: local configuration does not declare SSH key.\n'
+                  'Instances with connection type sshtun cannot launch.')
+        pkglogger.removeHandler(loghandler)
         if argv_parsed.become_daemon:
             if os.fork() != 0:
                 return 0
@@ -382,7 +388,14 @@ def main(argv=None):
             os.close(1)
             os.close(2)
         else:
-            logger.addHandler(logging.StreamHandler())
+            pkglogger.addHandler(logging.StreamHandler())
+        logfname = 'hardshare_client.{}.log'.format(config['wdeployments'][index]['id'])
+        loghandler = logging.FileHandler(filename=logfname, mode='a', delay=True)
+        loghandler.setLevel(logging.DEBUG)
+        loghandler.setFormatter(logging.Formatter('%(name)s.%(funcName)s (%(levelname)s) (pid: {});'
+                                                  ' %(asctime)s ; %(message)s'
+                                                  .format(os.getpid())))
+        pkglogger.addHandler(loghandler)
         return ac.run_sync(config['wdeployments'][index]['id'])
 
     elif argv_parsed.command == 'terminate':
@@ -648,6 +661,17 @@ def main(argv=None):
             config['wdeployments'][index]['init_inside'] = []
             modify_local(config)
 
+        elif argv_parsed.cprovider is not None:
+            selected_cprovider = argv_parsed.cprovider.lower()
+            if selected_cprovider not in ['docker', 'podman']:
+                print('ERROR: cprovider must be one of the following: docker, podman')
+                return 1
+            config, index, rc = get_config_with_index(argv_parsed.id_prefix)
+            if rc != 0:
+                return rc
+            config['wdeployments'][index]['cprovider'] = selected_cprovider
+            modify_local(config)
+
         elif argv_parsed.cprovider_img is not None:
             config, index, rc = get_config_with_index(argv_parsed.id_prefix)
             if rc != 0:
