@@ -201,6 +201,7 @@ class WorkspaceInstance:
             logger.info('found hostkey!')
             with open(hostkey_filename, 'rt', encoding='utf-8') as fp:
                 return fp.read().strip()
+            os.unlink(hostkey_filename)
         logger.info('did not find hostkey before timeout of {} s'.format(timeout))
         return None
 
@@ -462,7 +463,7 @@ class WorkspaceInstance:
                                                      stderr=subprocess.DEVNULL)
             rc = await p.wait()
             if rc != 0:
-                raise Exception('command ({}) returen error code: {}'.format(launch_args, rc))
+                raise Exception('command ({}) return error code: {}'.format(launch_args, rc))
 
             if self.cprovider == 'docker':
                 self.container_addr = await self.get_container_addr(timeout=10)
